﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.IO;
using System.Globalization;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

////////////////////////////////////////////////////////////////////////////////
// 小さい図形 (線分/連続線) を削除するアドイン
// 指定した値より、小さい図形 (線分/連続線) を削除します。
////////////////////////////////////////////////////////////////////////////////

namespace RCAddInSmallCleanCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        string CleanSmallShapesCommandName = "RootPro.AddIn.CreanSmallShapeCommand";     //コマンド名 

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

			CommandManager.AddMacroCommand("RCAddInSmallCleanCS", Properties.Resources.CommandName, CleanSmallShapes);

            //結合コマンドを追加する
            Command command = CommandManager.AddCommand(CleanSmallShapesCommandName, Properties.Resources.CommandName);

            command.ParameterGroups.Add(ParameterItemType.Double, 0, Properties.Resources.ThresholdParameterName);
            DoubleParameterItem doubleParameterItem = (DoubleParameterItem)command.GetParameterItemByID(0);
            doubleParameterItem.Description = Properties.Resources.ThresholdParameterDescription;
            doubleParameterItem.ValueSaveMode = ParameterValueSaveMode.OneTime;
            //            doubleParameterItem.Value = 2;
            //            doubleParameterItem.ValueSaveMode = ParameterValueSaveMode.Always;

            //開始イベントと終了イベントの追加
            command.Begin += new CommandBeginEventHandler(command_Begin);
            command.End += new EventHandler(command_End);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            CommandManager.RemoveMacroCommand(CleanSmallShapes);
            CommandManager.RemoveCommand(CleanSmallShapesCommandName);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// マクロコマンドの実行
        /// </summary>
        private void CleanSmallShapes()
        {
            //コマンドの実行
            CommandManager.ExecuteCommand(CleanSmallShapesCommandName);
        }

        /// <summary>
        /// コマンドの開始時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_Begin(Object sender, CommandBeginEventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの追加
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged);
        }

        /// <summary>
        /// コマンドの終了時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_End(Object sender, EventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの削除
            command.ParameterChanged -= new CommandParameterChangedEventHandler(command_ParameterChanged);
        }


        /// <summary>
        /// 分割割付を実行する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged(Object sender, CommandParameterChangedEventArgs e)
        {
            Document doc = ActiveDocument;         //ドキュメントの取得

            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID
            ParameterItem parameterItem = command.GetParameterItemByID(id); // パラメータアイテム

            //分割数の取得
            DoubleParameterItem doubleParameterItem = (DoubleParameterItem)parameterItem;
            if (doubleParameterItem.IsEmpty)
                return;
            double cleanValue = doubleParameterItem.Value;
            if (cleanValue > 100.0)
            {
                MessageBox.Show(Properties.Resources.ConfirmMessage1);
                e.CommandState = CommandState.ParameterError;
                return;
            }


            // UNDO 可能な操作を開始
            doc.UndoManager.BeginUndoUnit();

            //分割図形の作成
            int deleteCount = DoCleanSmallShapes(cleanValue);

            e.CommandState = CommandState.Execute;

            // UNDO 可能な操作を終了
            doc.UndoManager.EndUndoUnit();

            //結果の表示
            MessageBox.Show(string.Format(Properties.Resources.ConfirmMessage2, deleteCount), Properties.Resources.CommandName, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// 小さい図形の削除の実行
        /// </summary>
        /// <param name="cleanValue"></param>
        /// <returns></returns>
        private int DoCleanSmallShapes(double cleanValue)
        {
            int deleteCount = 0;
            Document doc = ActiveDocument;			//ドキュメントの取得
            int count = doc.Drawings.Count;
            for (int i = 0; i < count; i++)
            {
                Drawing drawing = doc.Drawings[i];

                ArrayList deleteShapeArray = new ArrayList();
                //図形を列挙する
                int shapeCount = drawing.Shapes.Count;
                Shape shape = null;
                while ((shape = drawing.Shapes.GetNextShape(shape)) != null)
                {
                    //結果フォームに追加
                    if (IsCleanShape(shape, cleanValue))
                    {
                        deleteShapeArray.Add(shape);        //削除リストに追加
                    }
                }

                foreach (Shape deleteShape in deleteShapeArray)
                {
                    //図形の削除
                    drawing.Shapes.Remove(deleteShape);
                    deleteCount++;
                }
            }

            return deleteCount;
        }


        /// <summary>
        /// 削除する図形か検査
        /// </summary>
        /// <param name="shape"></param>
        /// <param name="cleanValue"></param>
        /// <returns></returns>
        private bool IsCleanShape(Shape shape, double cleanValue)
        {
            if (shape is LineShape)
            {
                return ((LineShape)shape).Length < cleanValue;
            }
            else if (shape is PolylineShape)
            {
                return ((PolylineShape)shape).Length < cleanValue;
            }
            else if (shape is CircleShape)
            {
                return ((CircleShape)shape).Radius < cleanValue;
            }
            else if (shape is ArcShape)
            {
                return ((ArcShape)shape).Radius < cleanValue;
            }
            else if (shape is EllipseShape)
            {
                return ((EllipseShape)shape).MajorRadius < cleanValue;
            }
            else if (shape is EllipseArcShape)
            {
                return ((EllipseArcShape)shape).MajorRadius < cleanValue;
            }

            return false;
        }

		private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
		{
			AssemblyName assemblyName = new AssemblyName(args.Name);

			foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
			{
				if (AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
					return assembly;
			}

			if (assemblyName.Name.EndsWith(".resources"))
			{
				if (assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
				{
					string dllPath = Path.GetDirectoryName(this.AddInFullPath);
					dllPath = Path.Combine(dllPath, assemblyName.CultureName);
					dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

					if (File.Exists(dllPath))
					{
						try
						{
							return Assembly.LoadFrom(dllPath);
						}
						catch
						{
						}
					}
				}
			}

			return null;
		}
	}
}
